-----------------------------------------------------------------
----- LOCALIZED FUNCTIONS ---------------------------------------
-----------------------------------------------------------------
local math_sin, math_cos, sins, coss, network_player_from_global_index, obj_mark_for_deletion, obj_spawn_loot_yellow_coins,
 network_send_object, mmx_play_sound, obj_has_behavior_id, calculate_yaw, network_init_object,
 nearest_mario_state_to_object, obj_get_nearest_object_with_behavior_id, obj_check_hitbox_overlap, obj_scale, collision_find_surface_on_ray =
 math.sin, math.cos, sins, coss, network_player_from_global_index, obj_mark_for_deletion, obj_spawn_loot_yellow_coins,
  network_send_object, mmx_play_sound, obj_has_behavior_id, calculate_yaw, network_init_object,
  nearest_mario_state_to_object, obj_get_nearest_object_with_behavior_id, obj_check_hitbox_overlap, obj_scale, collision_find_surface_on_ray


local SHOT_WEAK = 1
local SHOT_MEDIUM = 2
local SHOT_STRONG = 3
local SHOT_GIGA_MAIN = 4
local SHOT_LIGHT_SMALL = 5
local SHOT_GIGA_SMALL = 6

local SHOT_MISS = 0
local SHOT_HIT_NO_DEATH = 1
local SHOT_HIT_DEATH = 2

projectile_active_hits = {}

-----------------------------------------------------------------
-----------------------------------------------------------------
----- INTERACTIONS ----------------------------------------------
-----------------------------------------------------------------
-----------------------------------------------------------------

function spawn_shot_fx(o, shot_pos, clank)
    if (clank == 1) then
        mmx_play_sound(SOUND_CLANK_HIT, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 0.4)  
    else
        mmx_play_sound(SOUND_SHOT_HIT, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 0.5)
    end
    local posx = shot_pos.x - (shot_pos.x - o.oPosX) / 2
    local posy = shot_pos.y - (shot_pos.y - o.oPosY) / 2
    local posz = shot_pos.z - (shot_pos.z - o.oPosZ) / 2
    spawn_non_sync_object(id_bhvWallspark, E_model_shot_dmg, posx , posy , posz , function (obj)
        obj.oMoveAngleYaw = calculate_yaw(shot_pos, {x = o.oPosX, y = o.oPosY, z = o.oPosZ})
    end)
end

function spawn_destroy_explosion(o, size)
    spawn_non_sync_object(id_bhvWhitePuffSmoke2, E_MODEL_EXPLOSION, o.oPosX , o.oPosY + 20 * size, o.oPosZ , function(obj)
        obj_scale(obj, size)
    end)
    mmx_play_sound(SOUND_SHOT_DESTROY, {x = o.oPosX, y = o.oPosY, z = o.oPosZ}, 0.7)
end

-------------------
--GENERAL ENEMIES--
-------------------

function shot_interact_srnormal(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if (o.oInteractStatus == ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED) then
        return SHOT_MISS
    end
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    o.oInteractStatus = ATTACK_PUNCH | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
    return SHOT_HIT_DEATH
end

function shot_interact_normal(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if (o.oInteractStatus == ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED) then
        return SHOT_MISS
    end
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    o.oInteractStatus = ATTACK_PUNCH | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_DEATH
end

function shot_interact_chuckya(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1.2)
        o.usingObj = nil
        obj_mark_for_deletion(o)
        obj_spawn_loot_yellow_coins(o, 5, 20.0)
        spawn_mist_particles_with_sound(SOUND_OBJ_CHUCKYA_DEATH)
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_enemygeneral(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if (o.oMmxHealth <= 0) or (o.oInteractStatus == ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED) then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oInteractStatus = ATTACK_GROUND_POUND_OR_TWIRL | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_bigenemygeneral(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if (o.oMmxHealth <= 0) or (o.oInteractStatus == ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED) then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1.2)
        o.oMmxHealth = o.oMmxInitialHealth
        o.oInteractStatus = ATTACK_GROUND_POUND_OR_TWIRL | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

-----------------------
--BOB-OMB BATTLEFIELD--
-----------------------

function shot_interact_koopa(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if (o.oMmxHealth <= 0) or (o.oInteractStatus == ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED)
    or o.oKoopaMovementType >= KOOPA_BP_KOOPA_THE_QUICK_BASE then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oMmxHealth = o.oMmxInitialHealth
        o.oInteractStatus = ATTACK_GROUND_POUND_OR_TWIRL | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_bobomb(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == BOBOMB_ACT_EXPLODE then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oAction = BOBOMB_ACT_EXPLODE
        o.oMmxHealth = o.oMmxInitialHealth
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_chainchomp(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 2)
        spawn_mist_particles_with_sound(SOUND_GENERAL_WALL_EXPLOSION)
        spawn_triangle_break_particles(20, 138, 3, 4);
        obj_mark_for_deletion(o)
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_waterbomb(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oAction = WATER_BOMB_ACT_EXPLODE
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_kingbobomb(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if (o.oFlags & OBJ_FLAG_HOLDABLE) == 0 or o.oAction == 8 or o.oAction == 7 then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1.5)
        o.oAction = 7
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_breakablebox(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1.2)
        o.oInteractStatus =  ATTACK_PUNCH | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_breakableboxsmall(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oInteractStatus = ATTACK_KICK_OR_TRIP | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED | INT_STATUS_STOP_RIDING
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_exclamationbox(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == 5 or o.oAction == 6 or o.oAction == 3 or o.oAction == 1 then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oInteractStatus = ATTACK_KICK_OR_TRIP | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
        o.oMmxHealth = o.oMmxInitialHealth
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

--------------------
--WHOMP'S FORTRESS--
--------------------

function shot_interact_breakablewall(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oMmxHealth <= 0 then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1.5)
        spawn_mist_particles_with_sound(SOUND_GENERAL_WALL_EXPLOSION)
        --obj_explode_and_spawn_coins(80, 0)
        obj_mark_for_deletion(o)
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_bridge(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if not (o.oFaceAnglePitch > -0x4000) then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1.2)
        o.oAction = 2
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_whomp(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1.2)
        o.oNumLootCoins = 5
        obj_spawn_loot_yellow_coins(o, 5, 20.0)
        o.oAction = 8
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_whompking(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == 8 or o.oAction == 0 or o.oAction == 9 then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 2)
        play_sound(SOUND_OBJ2_WHOMP_SOUND_SHORT, o.header.gfx.cameraToObject)
        play_sound(SOUND_OBJ_KING_WHOMP_DEATH, o.header.gfx.cameraToObject)
        o.oHealth = 0
        o.oAction = 8
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_piranhaplant(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == PIRANHA_PLANT_ACT_SHRINK_AND_DIE 
    or o.oAction == PIRANHA_PLANT_ACT_ATTACKED or o.oAction == PIRANHA_PLANT_ACT_WAIT_TO_RESPAWN then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        stop_secondary_music(50)
        o.oAction = PIRANHA_PLANT_ACT_ATTACKED
        o.oMmxHealth = o.oMmxInitialHealth
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_bulletbill(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == 3 or o.oAction == 4 then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1.2)
        o.oAction = 3
        spawn_non_sync_object(id_bhvExplosion, E_MODEL_EXPLOSION, o.oPosX, o.oPosY, o.oPosZ, nil)
        o.oMmxHealth = o.oMmxInitialHealth
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

----------------------
--COOL COOL MOUNTAIN--
----------------------

function shot_interact_mrblizzard(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == MR_BLIZZARD_ACT_DEATH then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oAction = MR_BLIZZARD_ACT_DEATH
        o.oMmxHealth = o.oMmxInitialHealth
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

----------------------
--BIG BOO'S HUNT------
----------------------
function shot_interact_boo(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if (o.oMmxHealth <= 0) or (o.oInteractStatus == ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED) then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        if o.oHealth > 1 then
            o.oMmxHealth = o.oMmxInitialHealth
        end
        o.oAction = 3
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

----------------------
--SHIFTING SAND LAND--
----------------------
function shot_interact_klepto(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oInteractStatus == ATTACK_FROM_ABOVE | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED | INT_STATUS_STOP_RIDING then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oInteractStatus = ATTACK_FROM_ABOVE | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED | INT_STATUS_STOP_RIDING
        o.oMmxHealth = o.oMmxInitialHealth
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_jumping_box(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oInteractStatus = INT_STATUS_STOP_RIDING
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_eyerokhand(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == EYEROK_HAND_ACT_DIE or o.oAction == EYEROK_HAND_ACT_DEAD then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oInteractStatus = INT_STATUS_STOP_RIDING
        o.parentObj.oEyerokBossNumHands = o.parentObj.oEyerokBossNumHands - 1
        o.oAction = EYEROK_HAND_ACT_DIE
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

--------------------
--LETHAL LAVA LAND--
--------------------
function shot_interact_mri(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == 3 then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oAction = 3
        o.oTimer = 104
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_bully(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMoveAngleYaw = shot_yaw
    spawn_shot_fx(o, shot_pos, 1)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if shot_dmg < 1 then
        o.oInteractStatus = INT_STATUS_INTERACTED
        o.oForwardVel = 15
    elseif shot_dmg >= 1 and shot_dmg < 4 then
        o.oInteractStatus = ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
        o.oForwardVel = 30
    else
        o.oInteractStatus = ATTACK_GROUND_POUND_OR_TWIRL | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
        o.oForwardVel = 50
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

---------------------
---TINY HUGE ISLAND--
---------------------

function shot_interact_spiny(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        spawn_mist_particles()
        play_sound(SOUND_OBJ_DEFAULT_DEATH, o.header.gfx.cameraToObject)
        o.oNumLootCoins = 1
        obj_spawn_loot_yellow_coins(o, 1, 20.0);
        obj_mark_for_deletion(o)
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_firepiranhaplant(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == FIRE_PIRANHA_PLANT_ACT_HIDE then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oInteractStatus = ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
        o.oMmxHealth = o.oMmxInitialHealth
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_wigglerhead(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oIntangibleTimer < 0 then
        return false
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oInteractStatus = ATTACK_FROM_ABOVE | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED | INT_STATUS_STOP_RIDING
        o.oMmxHealth = o.oMmxInitialHealth
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

------------------------
---TALL TALL MOUNTAIN---
------------------------

function shot_interact_monty_mole(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == MONTY_MOLE_ACT_SELECT_HOLE and 
    o.oInteractStatus == ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_destroy_explosion(o, 1)
        o.oInteractStatus = ATTACK_FAST_ATTACK | INT_STATUS_INTERACTED | INT_STATUS_WAS_ATTACKED
        o.oMmxHealth = o.oMmxInitialHealth
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

----------------------------
--BOWSER IN THE DARK WORLD--
----------------------------

function shot_interact_bowser(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    if o.oAction == 19 or o.oAction == 4 then
        return SHOT_MISS
    end
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        o.oHealth = o.oHealth - 1
        spawn_destroy_explosion(o, 1.8)
        if o.oHealth <= 0 then
            o.oAction = 4
        else
            o.oMmxHealth = o.oMmxInitialHealth
            o.oAction = 12
        end
        o.oMoveFlags = 0
        o.oForwardVel = 0
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end

function shot_interact_bowser_bomb(o, shot_syncid, shot_dmg, shot_pos, playerIndex, shot_yaw)
    o.oMmxHealth = o.oMmxHealth - shot_dmg
    spawn_shot_fx(o, shot_pos, 0)
    if projectile_active_hits[shot_syncid] ~= nil and not projectile_active_hits[shot_syncid][o.oSyncID] then
        projectile_active_hits[shot_syncid][o.oSyncID] = true
    end
    if (o.oMmxHealth <= 0) then
        spawn_non_sync_object(id_bhvExplosion, E_MODEL_EXPLOSION, o.oPosX, o.oPosY, o.oPosZ, nil)
        obj_mark_for_deletion(o)
        if o.oSyncID ~= 0 then
            network_send_object(o, false)
        end
        return SHOT_HIT_DEATH
    end
    if o.oSyncID ~= 0 then
        network_send_object(o, false)
    end
    return SHOT_HIT_NO_DEATH
end


-----------------------------------------------------------------
-----------------------------------------------------------------
----- INTERACTIONS HANDLING -------------------------------------
-----------------------------------------------------------------
-----------------------------------------------------------------

projectilehurtableenemy = {
    -- BOB
    [id_bhvGoomba] = shot_interact_enemygeneral, 
    [id_bhvKoopa] = shot_interact_koopa,
    [id_bhvBobomb] = shot_interact_bobomb,
    [id_bhvChainChomp] = shot_interact_chainchomp,
    [id_bhvChainChompGate] = shot_interact_chainchomp,
    [id_bhvWaterBomb] = shot_interact_waterbomb,
    [id_bhvKingBobomb] = shot_interact_kingbobomb,
    [id_bhvBreakableBox] = shot_interact_breakablebox,
    [id_bhvBreakableBoxSmall] = shot_interact_breakableboxsmall,
    [id_bhvExclamationBox] = shot_interact_exclamationbox,
    [id_bhvBowlingBall] = shot_interact_chainchomp,
    [id_bhvPitBowlingBall] = shot_interact_chainchomp,
    [id_bhvFreeBowlingBall] = shot_interact_chainchomp,
    -- WF
    [id_bhvKickableBoard] = shot_interact_bridge,
    [id_bhvWfBreakableWallLeft] = shot_interact_breakablewall,
    [id_bhvWfBreakableWallRight] = shot_interact_breakablewall,
    [id_bhvSmallWhomp] = shot_interact_whomp,
    [id_bhvWhompKingBoss] = shot_interact_whompking,
    [id_bhvPiranhaPlant] = shot_interact_piranhaplant,
    [id_bhvBulletBill] = shot_interact_bulletbill,
    -- CM
    [id_bhvSpindrift] = shot_interact_enemygeneral,
    [id_bhvMrBlizzard] = shot_interact_mrblizzard,
    -- BBH
    [id_bhvBoo] = shot_interact_boo,
    [id_bhvGhostHuntBoo] = shot_interact_boo,
    [id_bhvFlyingBookend] = shot_interact_enemygeneral,
    [id_bhvHauntedChair] = shot_interact_enemygeneral,
    [id_bhvBooWithCage] = shot_interact_boo,
    [id_bhvBookSwitch] = shot_interact_normal,
    [id_bhvMerryGoRoundBoo] = shot_interact_boo,
    [id_bhvBalconyBigBoo] = shot_interact_boo,
    [id_bhvGhostHuntBigBoo] = shot_interact_boo,
    [id_bhvMerryGoRoundBigBoo] = shot_interact_boo,
    [id_bhvMadPiano] = shot_interact_chainchomp,
    -- LLL
    [id_bhvMrI] = shot_interact_mri,
    [id_bhvSmallBully] = shot_interact_bully,
    [id_bhvBigBully] = shot_interact_bully,
    [id_bhvBigBullyWithMinions] = shot_interact_bully,
    -- SSL
    [id_bhvPokey] = shot_interact_enemygeneral,
    [id_bhvPokeyBodyPart] = shot_interact_enemygeneral,
    [id_bhvKlepto] = shot_interact_klepto,
    [id_bhvFlyGuy] = shot_interact_enemygeneral,
    [id_bhvEyerokHand] = shot_interact_eyerokhand,
    [id_bhvJumpingBox] = shot_interact_jumping_box,
    -- HMC
    [id_bhvScuttlebug] = shot_interact_enemygeneral,
    [id_bhvSnufit] = shot_interact_enemygeneral,
    [id_bhvSwoop] = shot_interact_enemygeneral,
    [id_bhvBigBoulder] = shot_interact_chainchomp,
    -- THI
    [id_bhvFirePiranhaPlant] = shot_interact_firepiranhaplant,
    [id_bhvChuckya] = shot_interact_chuckya,
    [id_bhvEnemyLakitu] = shot_interact_enemygeneral,
    [id_bhvSpiny] = shot_interact_spiny,
    [id_bhvWigglerHead] = shot_interact_wigglerhead,
    -- TTM
    [id_bhvMontyMole] = shot_interact_monty_mole,
    -- WDW
    [id_bhvSkeeter] = shot_interact_enemygeneral,
    -- SL
    [id_bhvSmallChillBully] = shot_interact_bully,
    [id_bhvBigChillBully] = shot_interact_bully,
    [id_bhvMoneybag] = shot_interact_enemygeneral,
    -- BOWSER
    [id_bhvBowser] = shot_interact_bowser,
    [id_bhvBowserBomb] = shot_interact_bowser_bomb,
}

function projectileattack(obj, shot_dmg, shot_pos, playerIndex, shot_yaw)
    local hurtenemy = SHOT_MISS
    local enemyobj

    local actualtable = projectilehurtableenemy
    -- Detect if SR enemies are present
    if (bhvSMSRShyGuy) then
        actualtable[bhvSMSRSmallBee] = shot_interact_srnormal
        actualtable[bhvSMSRShyGuy] = shot_interact_srnormal
    end
    
    for key,value in pairs(actualtable) do --actualtable is a table of enemies that can be hurt by projectiles with each enemy stored by behaviorid
        enemyobj = obj_get_nearest_object_with_behavior_id(obj,key) --get nearest hitable obj
        if enemyobj ~= nil and obj_check_hitbox_overlap(obj, enemyobj) then --check if obj is touching enemy obj
            if actualtable[key] and (projectile_active_hits[obj.oSyncID] ~= nil and not projectile_active_hits[obj.oSyncID][enemyobj.oSyncID]) then
                hurtenemy = actualtable[key](enemyobj, obj.oSyncID, shot_dmg, shot_pos, playerIndex, shot_yaw)
            end
        end
    end
    return hurtenemy
end

function projectile_hit_other_players(obj)
    local np = network_player_from_global_index(obj.oOwner)
    local m = gMarioStates[np.localIndex]
    local player = nearest_mario_state_to_object(obj)
    if player ~= nil and obj_check_hitbox_overlap(obj, player.marioObj) and player.playerIndex ~= m.playerIndex then
        return true
    end
    return false
end

function spawn_shot(m, armHeight, armLength, armTheta, chargeLevel)
    local mGfx = m.marioObj.header.gfx
    local s = gPlayerSyncTable[m.playerIndex]
    if chargeLevel == 0 then
        local busterSpawnTheta = ((m.faceAngle.y + armTheta) / 0x8000) * math.pi
        local busterXOffset =  math_sin(busterSpawnTheta) * armLength
        local busterZOffset =  math_cos(busterSpawnTheta) * armLength
        if (s.character == 'x') then
            if (m.playerIndex == 0) then
                mmx_play_sound(SOUND_X_SHOT1, m.pos, 0.45)
                network_send(true, {type = 'shot_sound', shot_idx = 1, intensity = 0.73, m = gNetworkPlayers[m.playerIndex].globalIndex})
                s.x_active_shots = s.x_active_shots + 1
                spawn_sync_object(id_bhvShot, E_model_shot1, mGfx.pos.x + busterXOffset, mGfx.pos.y + armHeight, mGfx.pos.z + busterZOffset, function(obj)
                    obj.oMoveAngleYaw = m.faceAngle.y
                    obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                    obj.oShotDestroyNextFrame = 0
                    obj.oOwnerChar = 0
                    obj.oShotType = SHOT_WEAK
                    obj.oMmxAtkDmg = 0.95
                end)
            end
        elseif (s.character == 'vile') then
            mmx_play_sound(SOUND_VILE_SHOT, m.pos, 0.44)
            if m.playerIndex == 0 then
                spawn_sync_object(id_bhvShot, E_model_vulcan_shot, mGfx.pos.x + busterXOffset, mGfx.pos.y + armHeight, mGfx.pos.z + busterZOffset, function(obj)
                    obj.oMoveAngleYaw = m.faceAngle.y
                    obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                    obj.oShotDestroyNextFrame = 0
                    obj.oOwnerChar = 1
                    obj.oShotType = SHOT_WEAK
                    obj.oMmxAtkDmg = 0.95
                end)
            end
        end
    elseif chargeLevel == 1 then
        if m.playerIndex == 0 then
            mmx_play_sound(SOUND_X_SHOT2, m.pos, 0.73)
            network_send(true, {type = 'shot_sound', shot_idx = 2, intensity = 0.73, m = gNetworkPlayers[m.playerIndex].globalIndex})
            local busterSpawnTheta = ((m.faceAngle.y + armTheta) / 0x8000) * math.pi
            local busterXOffset =  math_sin(busterSpawnTheta) * armLength
            local busterZOffset =  math_cos(busterSpawnTheta) * armLength
            spawn_sync_object(id_bhvShot, E_model_shot2, mGfx.pos.x + busterXOffset, mGfx.pos.y + armHeight, mGfx.pos.z + busterZOffset, function(obj)
                obj.oMoveAngleYaw = m.faceAngle.y
                obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                obj.oShotDestroyNextFrame = 0
                obj.oOwnerChar = 0
                obj.oShotType = SHOT_MEDIUM
                obj.oMmxAtkDmg = 3.8
            end)
        end
    elseif chargeLevel == 2 then
        local busterSpawnTheta = ((m.faceAngle.y + armTheta) / 0x8000) * math.pi
        local busterXOffset =  math_sin(busterSpawnTheta) * armLength
        local busterZOffset =  math_cos(busterSpawnTheta) * armLength
        if (s.character == 'x') then
            if m.playerIndex == 0 then
                mmx_play_sound(SOUND_X_SHOT3, m.pos, 0.73)
                network_send(true, {type = 'shot_sound', shot_idx = 3, intensity = 0.73, m = gNetworkPlayers[m.playerIndex].globalIndex})
                spawn_sync_object(id_bhvShot, E_model_shot3, mGfx.pos.x + busterXOffset, mGfx.pos.y + armHeight, mGfx.pos.z + busterZOffset, function(obj)
                    obj.oMoveAngleYaw = m.faceAngle.y
                    obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                    obj.oShotDestroyNextFrame = 0
                    obj.oOwnerChar = 0
                    obj.oShotType = SHOT_STRONG
                    if (s.x_armor == 0) then
                        obj.oMmxAtkDmg = 8
                    else
                        obj.oMmxAtkDmg = 6.2
                    end
                end)
            end
        elseif (s.character == 'vile') then
            mmx_play_sound(SOUND_VILE_CANNON, m.pos, 0.54)
            if m.playerIndex == 0 then
                spawn_sync_object(id_bhvShot, E_model_cannon_shot, mGfx.pos.x + busterXOffset, mGfx.pos.y + armHeight, mGfx.pos.z + busterZOffset, function(obj)
                    obj.oMoveAngleYaw = m.faceAngle.y
                    obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                    obj.oShotDestroyNextFrame = 0
                    obj.oOwnerChar = 1
                    obj.oShotType = SHOT_STRONG
                    obj.oMmxAtkDmg = 5.5
                end)
            end
        end
    elseif chargeLevel == 3 then
        if m.playerIndex == 0 then
            mmx_play_sound(SOUND_X_SHOT_MAX, m.pos, 0.9)
            network_send(true, {type = 'shot_sound', shot_idx = 4, intensity = 0.9, m = gNetworkPlayers[m.playerIndex].globalIndex})
            local busterSpawnTheta = ((m.faceAngle.y + armTheta) / 0x8000) * math.pi
            local busterSpawnTheta2 = ((m.faceAngle.y + armTheta - 10000) / 0x8000) * math.pi
            local busterXOffset =  math_sin(busterSpawnTheta) * armLength
            local busterZOffset =  math_cos(busterSpawnTheta) * armLength
            local busterXOffset2 =  sins(m.faceAngle.y + 16384) * 50
            local busterZOffset2 =  coss(m.faceAngle.y + 16384) * 50
            spawn_sync_object(id_bhvShot, E_model_shot_max, mGfx.pos.x + busterXOffset, mGfx.pos.y + armHeight, mGfx.pos.z + busterZOffset, function(obj)
                obj.oMoveAngleYaw = m.faceAngle.y
                obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                obj.oShotDestroyNextFrame = 0
                obj.oOwnerChar = 0
                obj.oChargeRot = 0
                obj.oShotType = SHOT_LIGHT_SMALL
                obj.oMmxAtkDmg = 5.8
            end)
            spawn_sync_object(id_bhvShot, E_model_shot_max, mGfx.pos.x + busterXOffset, mGfx.pos.y + armHeight - 15, mGfx.pos.z + busterZOffset, function(obj)
                obj.oMoveAngleYaw = m.faceAngle.y
                obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                obj.oShotDestroyNextFrame = 0
                obj.oOwnerChar = 0
                obj.oShotOffset = math.pi
                obj.oChargeRot = 0
                obj.oShotType = SHOT_LIGHT_SMALL
                obj.oMmxAtkDmg = 5.8
            end)
            spawn_sync_object(id_bhvShot, E_model_shot_max, mGfx.pos.x + busterXOffset - busterXOffset2,
             mGfx.pos.y + armHeight, mGfx.pos.z + busterZOffset - busterZOffset2, function(obj)
                obj.oMoveAngleYaw = m.faceAngle.y
                obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                obj.oShotDestroyNextFrame = 0
                obj.oOwnerChar = 0
                obj.oChargeRot = 1
                obj.oShotType = SHOT_LIGHT_SMALL
                obj.oMmxAtkDmg = 5.8
            end)
        end
    elseif chargeLevel == 4 then
        if m.playerIndex == 0 then
            mmx_play_sound(SOUND_X_SHOT_MAX, m.pos, 0.9)
            network_send(true, {type = 'shot_sound', shot_idx = 4, intensity = 0.9, m = gNetworkPlayers[m.playerIndex].globalIndex})
            local busterXOffset = sins(m.faceAngle.y + armTheta) * armLength
            local busterZOffset = coss(m.faceAngle.y + armTheta) * armLength
            local busterXOffset2 = sins(m.faceAngle.y + armTheta + 8000) * armLength
            local busterZOffset2 = coss(m.faceAngle.y + armTheta + 8000) * armLength
            spawn_sync_object(id_bhvShot, E_model_giga_pink_shot, mGfx.pos.x + busterXOffset, mGfx.pos.y + armHeight, mGfx.pos.z + busterZOffset, function(obj)
                obj.oMoveAngleYaw = m.faceAngle.y
                obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                obj.oShotDestroyNextFrame = 0
                obj.oShotDestroyNextFrame = 0
                obj.oOwnerChar = 0
                obj.oShotType = SHOT_GIGA_MAIN
                obj.oMmxAtkDmg = 7.5
            end)
            spawn_sync_object(id_bhvShot, E_model_shot3_particle, mGfx.pos.x + busterXOffset2, mGfx.pos.y + armHeight - 70, mGfx.pos.z + busterZOffset2, function(obj)
                obj.oMoveAngleYaw = m.faceAngle.y
                obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                obj.oShotDestroyNextFrame = 0
                obj.oOwnerChar = 0
                obj.oChargeRot = 0
                obj.oShotType = SHOT_GIGA_SMALL
                obj.oMmxAtkDmg = 5
            end)
        end
    end
end


-----------------------------------------------------------------
-----------------------------------------------------------------
----- BEHAVIORS -------------------------------------------------
-----------------------------------------------------------------
-----------------------------------------------------------------


local shot_init = function(o)
    network_init_object(o, true, { 'oSyncID' })
    o.oFlags = (OBJ_FLAG_UPDATE_GFX_POS_AND_ANGLE | OBJ_FLAG_SET_FACE_ANGLE_TO_MOVE_ANGLE | OBJ_FLAG_SET_FACE_YAW_TO_MOVE_YAW)
    projectile_active_hits[o.oSyncID] = {}
    local np = network_player_from_global_index(o.oOwner)
    local m = gMarioStates[np.localIndex]
    if o.oShotType == SHOT_WEAK then
        if o.oOwnerChar == 1 then
            obj_scale(o, 0.4)
        end
        if (m.playerIndex ~= 0) then
            o.oIntangibleTimer = 0
            if gServerSettings.playerInteractions == PLAYER_INTERACTIONS_PVP then
                o.oInteractType = INTERACT_DAMAGE
                o.oDamageOrCoinValue = 0
            end
        end
        o.hitboxRadius            = 30
        o.hitboxHeight            = 65
        o.hitboxDownOffset        = 50
        o.oWallHitboxRadius = 20
        o.oForwardVel = 92
    end
    if o.oShotType == SHOT_MEDIUM then
        o.header.gfx.scale.x = 1.15
        o.header.gfx.scale.z = 1.15
        o.header.gfx.scale.y = 1.15
        if (m.playerIndex ~= 0) then
            o.oIntangibleTimer = 0
            if gServerSettings.playerInteractions == PLAYER_INTERACTIONS_PVP then
                o.oInteractType = INTERACT_DAMAGE
                o.oDamageOrCoinValue = 1
            end
        end
        o.hitboxRadius            = 70
        o.hitboxHeight            = 90
        o.hitboxDownOffset        = 70
        o.oWallHitboxRadius = 60
        o.oForwardVel = 130
    end
    if o.oShotType == SHOT_STRONG or o.oShotType == SHOT_GIGA_MAIN then
        if (m.playerIndex ~= 0) then
            if gServerSettings.playerInteractions ~= PLAYER_INTERACTIONS_NONE then
                o.oInteractType = INTERACT_DAMAGE
                o.oIntangibleTimer = 0
                if gServerSettings.playerInteractions == PLAYER_INTERACTIONS_PVP then
                    o.oDamageOrCoinValue = 2
                end
            end
        end
        o.hitboxRadius            = 130
        o.hitboxHeight            = 160
        o.hitboxDownOffset        = 110
        o.oWallHitboxRadius = 120
        if (o.oOwnerChar == 0) then
            o.oForwardVel = 170
            spawn_non_sync_object(id_bhvShotRing, E_model_shot3_ring, o.oPosX, o.oPosY, o.oPosZ, function(obj)
                obj.oForwardVel = 60
                obj.oMoveAngleYaw = o.oMoveAngleYaw
            end)
        elseif (o.oOwnerChar == 1) then
            o.oForwardVel = 140
            o.hitboxRadius            = 120
            o.oWallHitboxRadius = 110
            o.hitboxHeight            = 150
            o.hitboxDownOffset        = 100
        end
    end
    if o.oShotType == SHOT_LIGHT_SMALL or o.oShotType == SHOT_GIGA_SMALL then
        o.oWallHitboxRadius = 10
        if (m.playerIndex ~= 0) then
            o.oInteractType = INTERACT_DAMAGE
            o.oIntangibleTimer = 0
            if gServerSettings.playerInteractions == PLAYER_INTERACTIONS_PVP then
                o.oDamageOrCoinValue = 2
            end
        end
        o.hitboxRadius            = 87
        o.oWallHitboxRadius = 77
        o.hitboxHeight            = 105
        o.hitboxDownOffset        = 80
        o.oForwardVel = 170
        if o.oShotType == SHOT_LIGHT_SMALL then
            spawn_non_sync_object(id_bhvCharge2Front, E_model_shot_max_bubble, o.oPosX, o.oPosY, o.oPosZ, function(obj)
                obj.oForwardVel = 60
                obj.oMoveAngleYaw = o.oMoveAngleYaw
                obj.oOwner = gNetworkPlayers[m.playerIndex].globalIndex
                obj.oMoveAngleYaw = o.oMoveAngleYaw
            end)
        end
        --spawn_non_sync_object(id_bhvShotRing, E_model_shot_max_ring, o.oPosX, o.oPosY + 40, o.oPosZ, function(obj)
        --    obj.oForwardVel = 60
        --    obj.oMoveAngleYaw = o.oMoveAngleYaw
        --end)
    end
    o.oVelX = sins(o.oMoveAngleYaw) * o.oForwardVel
    o.oVelY = 0
    o.oVelZ = coss(o.oMoveAngleYaw) * o.oForwardVel
    --obj_set_hitbox_radius_and_height(o, radius, height)
end

local shot_loop = function(o)
    local np = network_player_from_global_index(o.oOwner)
    local m = gMarioStates[np.localIndex]
    local projectileStep = projectileattack(o, o.oMmxAtkDmg, { x = o.oPosX, y = o.oPosY, z = o.oPosZ}, m.playerIndex, o.oMoveAngleYaw)
    if o.oShotDestroyNextFrame == 1 then
        projectile_active_hits[o.oSyncID] = nil
        if (m.playerIndex == 0) then
            local s = gPlayerSyncTable[0]
            if (s.character == 'x' and s.x_active_shots > 0) then
                s.x_active_shots = s.x_active_shots - 1
            end
        end
        obj_mark_for_deletion(o)
    end
    if projectileStep ~= SHOT_MISS then
        if (o.oShotType == SHOT_WEAK) or (o.oShotType ~= SHOT_WEAK and projectileStep == SHOT_HIT_NO_DEATH) then
            o.oShotDestroyNextFrame = 1
        end
    end
    local shot_speed
    local shot_life
    if o.oShotType == SHOT_WEAK then
        if (o.oOwnerChar == 0) then
            shot_life = 31
        elseif (o.oOwnerChar == 1) then
            shot_life = 25
        end
    elseif o.oShotType ==  SHOT_MEDIUM then
        shot_life = 37
    elseif o.oShotType == SHOT_STRONG or o.oShotType == SHOT_GIGA_MAIN then
        obj_scale(o, 1.4)
        shot_life = 45
        o.oMoveAngleRoll = o.oMoveAngleRoll + 4000 + 6000 / (o.oTimer * o.oTimer)
        if o.oTimer % 4 and o.oOwnerChar == 0 then
            local shotTheta = o.oMoveAngleYaw / 0x8000 * math.pi
            local randomY = math.random(-50, 50)
            local randomPitch = math.random(-4000, 4000)
            if o.oShotType == SHOT_STRONG then
                spawn_non_sync_object(id_bhvShotTrailBall, E_model_shot3_particle, o.oPosX - 40 * math_sin(shotTheta + randomPitch), o.oPosY + randomY, o.oPosZ - 40 * math_cos(shotTheta + randomPitch), function(obj)
                    obj.oForwardVel = 60
                    obj.oMoveAngleYaw = o.oMoveAngleYaw
                    obj.oShotType = 3
                end)
            else
                spawn_non_sync_object(id_bhvShotTrailBall, E_model_shot_max, o.oPosX - 40 * math_sin(shotTheta + randomPitch), o.oPosY + randomY, o.oPosZ - 40 * math_cos(shotTheta + randomPitch), function(obj)
                    obj.oForwardVel = 60
                    obj.oMoveAngleYaw = o.oMoveAngleYaw
                    obj.oShotType = 3
                end)
            end
        end
        if o.oTimer % 3 == 0 then
            o.header.gfx.scale.y = 1.3;
        elseif o.oTimer % 2 == 0 then
            o.header.gfx.scale.x = 1.3
            o.header.gfx.scale.z = 1.3
        end
    elseif o.oShotType == SHOT_LIGHT_SMALL then
        obj_scale(o, 1.4)
        shot_life = 50
        spawn_non_sync_object(id_bhvShotTrailBall, E_model_shot_max, o.oPosX, o.oPosY, o.oPosZ, function(obj)
            obj.oForwardVel = 60
            obj.oMoveAngleYaw = o.oMoveAngleYaw
            obj.oShotType = 4
        end)
    elseif o.oShotType == SHOT_GIGA_SMALL then
        obj_scale(o, 1.2)
        shot_life = 50
        spawn_non_sync_object(id_bhvShotTrailBall, E_model_shot3_particle, o.oPosX, o.oPosY, o.oPosZ, function(obj)
            obj.oForwardVel = 60
            obj.oMoveAngleYaw = o.oMoveAngleYaw
            obj.oShotType = 4
        end)
    end
    local prevPosX = o.oPosX
    local prevPosZ = o.oPosZ
    
    local collision = collision_find_surface_on_ray(
        o.oPosX, o.oPosY, o.oPosZ,
        o.oVelX, o.oVelY, o.oVelZ)
    
    --djui_chat_message_create(tostring(collide))
    if collision.surface ~= nil or (projectile_hit_other_players(o) and o.oInteractType ~= INTERACT_DAMAGE) then
        o.oShotDestroyNextFrame = 1
    else
        if (o.oShotType == SHOT_LIGHT_SMALL or o.oShotType == SHOT_GIGA_SMALL) then
            local ang = (o.oMoveAngleYaw / 0x8000) * math.pi
            if o.oChargeRot == 0 then
                if (o.oShotType == SHOT_LIGHT_SMALL) then
                    o.oPosX = o.oPosX + o.oVelX + 65 * math_sin(ang + o.oTimer - o.oShotOffset + math.pi / 2)
                    o.oPosY = o.oPosY + o.oVelY + 65 * math_sin(o.oTimer)
                    o.oPosZ = o.oPosZ + o.oVelZ + 65 * math_cos(ang + o.oTimer - o.oShotOffset + math.pi / 2)
                else
                    o.oPosX = o.oPosX + o.oVelX + 100 * math_sin(ang + o.oTimer - o.oShotOffset + math.pi / 2)
                    o.oPosY = o.oPosY + o.oVelY + 100 * math_sin(o.oTimer)
                    o.oPosZ = o.oPosZ + o.oVelZ + 100 * math_cos(ang + o.oTimer - o.oShotOffset + math.pi / 2)
                end
            else
                o.oPosX = o.oPosX + o.oVelX + 65 * math_sin(ang + o.oTimer - o.oShotOffset)
                o.oPosY = o.oPosY + o.oVelY + 65 * math_cos(o.oTimer)
                o.oPosZ = o.oPosZ + o.oVelZ + 65 * math_cos(ang + o.oTimer - o.oShotOffset)
            end
        else
            o.oPosX = o.oPosX + o.oVelX
            o.oPosY = o.oPosY + o.oVelY
            o.oPosZ = o.oPosZ + o.oVelZ
        end
    end

    if o.oTimer >= shot_life then
        projectile_active_hits[o.oSyncID] = nil
        if (m.playerIndex == 0) then
            local s = gPlayerSyncTable[0]
            if (s.character == 'x' and s.x_active_shots > 0) then
                s.x_active_shots = s.x_active_shots - 1
            end
        end
        obj_mark_for_deletion(o)
    end
end

----------------------------
--BEHAVIORS REGISTRY--------
----------------------------
id_bhvShot = hook_behavior(nil, OBJ_LIST_GENACTOR, false, shot_init, shot_loop)

hook_event(HOOK_ON_INTERACT, function(m, o, type, value)
    if obj_has_behavior_id(o, id_bhvShot) ~= 0 or obj_has_behavior_id(o, id_bhvLegNapalmExplosion) or obj_has_behavior_id(o, id_bhvZsaberHitBox) then
        if obj_has_behavior_id(o, id_bhvShot) ~= 0 then
            obj_mark_for_deletion(o)
        end
        if _G.HideAndSeek and o.oMmxAtkDmg > 1 then
            local np = network_player_from_global_index(o.oOwner)
            local mOwner = gMarioStates[np.localIndex]
            if _G.HideAndSeek.is_player_seeker(mOwner.playerIndex) and not _G.HideAndSeek.is_player_seeker(m.playerIndex) then
                _G.HideAndSeek.set_player_seeker(m.playerIndex, true)
            end
        end
    end
end)